package MyModel::Author;
use feature qw( signatures );
use Moose;
extends 'MyModel';
no warnings qw( experimental::signatures );

sub get_page ($self, $page_number = 1, $name_like = undef, $count_min = 0) {
   my $search_condition = (defined $name_like)
      ? { "me.name" => { ilike => "\%$name_like\%" } }
      : undef;
   my $rs = $self->schema->resultset('Author')->search_rs(
      $search_condition,
      {  join => 'books',
         select => [ 'me.id', 'me.name', { count => 'books.id', -as => 'bcount' } ],
         order_by => [ 'me.name' ],
         group_by => [ 'me.id', 'me.name' ],
         having => \[ 'count(books.id) >= ?', $count_min ],
         page => $page_number,
         rows => $self->rows_per_page,
      },
   );
   return $rs;
}

sub find_by_id ($self, $id, $page_number = 1) {
   my $author = $self->schema->resultset('Author')->find({ id => $id });
   return (undef, undef) if (!defined $author);
   my $book_page = $author->books_rs->search(
      undef,
      {  order_by => [ "me.title" ],
         page => $page_number,
         rows => $self->rows_per_page,
      },
   );
   return ($author, $book_page);
}

no Moose;
__PACKAGE__->meta->make_immutable;
